#
# Copyright 2014 CHLSoftware
#
# A monitor class using Watchdog DirectorySnapshot functions.
#
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import os
import shutil
import LwpMotoUtils
import urllib
from watchdog.utils.dirsnapshot import DirectorySnapshot,DirectorySnapshotDiff
from pathtools.path import absolute_path
from datetime import datetime

global DOCMOTO_BASE_FOLDER
global DOCMOTO_SERVER_URL
global DOCMOTO_USERNAME
global DOCMOTO_PWD
global LOG_MODE
global LOG_PATH
global REMOVE_AFTER_UPLOAD
global ARCHIVE_PATH


#A stub to make DirectorySnapshot happy. Otherwise un-used
def walker_callback(path, stat_info, self=None):
    return

#monitor class
class CHLMonitor():

    def __init__(self):
        self._path = None
        self._is_recursive = True
        self._oldSnapshot = None
        self._newSnapshot = None
        self._diffSnapshot = None
        self._is_started = False
        self._root = DOCMOTO_SERVER_URL + "/" + DOCMOTO_BASE_FOLDER

    def watch(self,path,recursive=True):
        self._path = absolute_path(path)
        self._is_recursive = recursive

    def scan(self):
        if not self._is_started:
            self._is_started = True
            self._newSnapshot = DirectorySnapshot( path=self._path,
                                                   recursive=self._is_recursive,
                                                   walker_callback=walker_callback)
            self._oldSnapshot = self._newSnapshot
        else:
            self._newSnapshot = DirectorySnapshot(path=self._path,
                                                  recursive=self._is_recursive,
                                                  walker_callback=walker_callback)
            self._diffSnapshot = DirectorySnapshotDiff(self._oldSnapshot,self._newSnapshot)
            self._oldSnapshot = self._newSnapshot
            self.process_diff()
        return

    def process_diff(self):

        #Connect to DocMoto
        myRet = LwpMotoUtils.secureinitialize(DOCMOTO_USERNAME,DOCMOTO_PWD,DOCMOTO_SERVER_URL)

        if not myRet['returnOK']:
            self.writeLog("DocMoto connection error " + str(myRet['responseString']))
            return
        self.process_dirs()
        self.process_files()

        return

    def process_dirs(self):
        for my_path in self._diffSnapshot.dirs_created:
            myURL = self._root

            #Remove unwanted part of path
            my_path = my_path.replace(self._path,'')

            #Check each dir en route
            my_dir_list = my_path.split("/")

            for my_dir in my_dir_list:
                if len(my_dir) > 0:

                    myURL = myURL + "/" + urllib.quote(str(my_dir))

                    myRet = LwpMotoUtils.checkExists(myURL)

                    if(not myRet['exists']):
                        LwpMotoUtils.createCollection(myURL)
                        if LOG_MODE > 0:
                            self.writeLog("Dir created " + myURL)

    def process_files(self):
        for my_path in self._diffSnapshot.files_created:
            myURL = self._root
            myFullFileName = my_path

            #Remove unwanted part of path to get the name only
            myFileName = my_path.replace(self._path,'')

            myURL = myURL + urllib.quote(str(myFileName))
            if self.sendFile(myURL,myFullFileName):
                if REMOVE_AFTER_UPLOAD == 1:
                    self.archiveFile(myFileName,myFullFileName)
                elif REMOVE_AFTER_UPLOAD == 2:
                    self.removeFile(myFullFileName)


    def sendFile(self,myURL,fileName):

        try:
            f = open(fileName,'rb')
            buff = f.read()
            f.close()
        except:
            self.writeLog("Unexpected sendFile return error for file " + fileName)
            return False

        #Quick look to see if exists
        myRet = LwpMotoUtils.checkExists(myURL)
        if(myRet['exists']):
            self.writeLog("File already exists error for file (will be ignored if on port 4983 or 4984) " + fileName)


        LwpMotoUtils.createPut(myURL,buff)

        # Add any properties. Correspond to DocMoto tags
        request = """<?xml version='1.0' encoding='utf-8'?>
                  <propertyupdate xmlns='DAV:' xmlns:dm='http://www.docmoto.com/2008/03/uiconfig'>
                    <set>
                      <prop>
                        <comment>Moved up by file monitor app</comment>
                      </prop>
                    </set>
                  </propertyupdate>"""

        # Confirm
        LwpMotoUtils.createPropPatch(myURL, request)

        LwpMotoUtils.createVersionControl(myURL)

        if LOG_MODE > 0:
            self.writeLog("File created " + fileName)
        return True


    def removeFile(self,fileName):
        try:
            os.remove(fileName)
        except:
            self.writeLog("Unexpected removeFile error for file " + fileName)
            return

    def archiveFile(self,fileName,fileNameAndPath):
        destFileNameAndPath = absolute_path(ARCHIVE_PATH) + fileName

        #Create any directories that don't exist in the archive
        if not os.path.exists(os.path.dirname(destFileNameAndPath)):
            os.makedirs(os.path.dirname(destFileNameAndPath))

        #Move it over
        shutil.move(fileNameAndPath,destFileNameAndPath)

    def writeLog(self,message):
         with open(LOG_PATH,'a+') as f:
            i = datetime.now()
            f.write(i.strftime('%Y/%m/%d %H:%M:%S') + ' ' + message + "\n")
